@@ -18,10 +18,15 @@
  * Authors: Carlos Garnacho Parro  <carlosg@gnome.org>
  */
 
+#include "config.h"
 #include <dbus/dbus.h>
 #include <glib-object.h>
-#include <libhal.h>
 #include <string.h>
+
+#ifdef HAVE_HAL
+#include <libhal.h>
+#endif
+
 #include "oobs-session-private.h"
 #include "oobs-list-private.h"
 #include "oobs-object-private.h"
@@ -33,7 +38,6 @@
 #include "oobs-iface-ppp.h"
 #include "iface-state-monitor.h"
 #include "utils.h"
-#include "config.h"
 
 /**
  * SECTION:oobs-ifacesconfig
@@ -78,29 +82,6 @@ static void oobs_ifaces_config_commit     (OobsObject   *object);
 
 G_DEFINE_TYPE (OobsIfacesConfig, oobs_ifaces_config, OOBS_TYPE_OBJECT);
 
-GType
-oobs_iface_type_get_type (void)
-{
-  static GType etype = 0;
-
-  if (!etype)
-    {
-      static const GEnumValue values[] =
-	{
-	  { OOBS_IFACE_TYPE_ETHERNET, "OOBS_IFACE_TYPE_ETHERNET", "ethernet" },
-	  { OOBS_IFACE_TYPE_WIRELESS, "OOBS_IFACE_TYPE_WIRELESS", "wireless" },
-	  { OOBS_IFACE_TYPE_IRLAN,    "OOBS_IFACE_TYPE_IRLAN",    "infrared" },
-	  { OOBS_IFACE_TYPE_PLIP,     "OOBS_IFACE_TYPE_PLIP",     "parallel" },
-	  { OOBS_IFACE_TYPE_PPP,      "OOBS_IFACE_TYPE_PPP",      "ppp" },
- 	  { 0, NULL, NULL }
-	};
-
-      etype = g_enum_register_static ("OobsIfaceType", values);
-    }
-
-  return etype;
-}
-
 static void
 oobs_ifaces_config_class_init (OobsIfacesConfigClass *class)
 {
@@ -184,11 +165,20 @@ hal_context_get_initial_devices (LibHalContext *context)
   GHashTable *devices;
   gint i, n_devices;
   gchar **udis;
+  DBusError error;
 
   devices = g_hash_table_new_full (g_str_hash, g_str_equal,
 				   (GDestroyNotify) g_free, NULL);
 
-  udis = libhal_find_device_by_capability (context, "net", &n_devices, NULL);
+  dbus_error_init(&error);
+  udis = libhal_find_device_by_capability (context, "net", &n_devices, &error);
+
+  /* In case an error occurred, udis would be NULL, possibly leading to crash */
+  if (dbus_error_is_set (&error)) {
+    g_warning ("Could not find any network device on the system: %s.",
+               error.message);
+    return devices;
+  }
 
   for (i = 0; i < n_devices; i++)
     g_hash_table_insert (devices, g_strdup (udis[i]), GINT_TO_POINTER (TRUE));
@@ -812,9 +802,14 @@ oobs_ifaces_config_commit (OobsObject *object)
 OobsObject*
 oobs_ifaces_config_get (void)
 {
-  return g_object_new (OOBS_TYPE_IFACES_CONFIG,
-		       "remote-object", IFACES_CONFIG_REMOTE_OBJECT,
-		       NULL);
+  static OobsObject *the_object = NULL;
+
+  if (!the_object)
+    the_object = g_object_new (OOBS_TYPE_IFACES_CONFIG,
+                               "remote-object", IFACES_CONFIG_REMOTE_OBJECT,
+                               NULL);
+
+  return the_object;
 }
 
 /**
