@@ -24,9 +24,17 @@
 #include "oobs-object-private.h"
 #include "oobs-session.h"
 #include "oobs-session-private.h"
+#include "oobs-error.h"
+#include "utils.h"
+
+/**
+ * SECTION:oobs-object
+ * @title: OobsObject
+ * @short_description: Base object for all configuration objects
+ * @see_also: #OobsList
+ **/
 
 #define OOBS_OBJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OOBS_TYPE_OBJECT, OobsObjectPrivate))
-#define POLKIT_ACTION "org.freedesktop.systemtoolsbackends.set"
 
 typedef struct _OobsObjectPrivate OobsObjectPrivate;
 typedef struct _OobsObjectAsyncCallbackData OobsObjectAsyncCallbackData;
@@ -54,6 +62,14 @@ struct _OobsObjectAsyncCallbackData
   gpointer data;
 };
 
+enum _OobsObjectCommitMethod
+{
+  METHOD_COMMIT,
+  METHOD_ADD,
+  METHOD_DELETE
+};
+typedef enum _OobsObjectCommitMethod _OobsObjectCommitMethod;
+
 static void oobs_object_class_init (OobsObjectClass *class);
 static void oobs_object_init       (OobsObject      *object);
 static void oobs_object_finalize   (GObject         *object);
@@ -71,8 +87,6 @@ static void oobs_object_get_property (GObject       *object,
 				      GValue        *value,
 				      GParamSpec    *pspec);
 
-static const gchar * oobs_object_real_get_authentication_action (OobsObject *object);
-
 static void connect_object_to_session (OobsObject *object);
 
 enum
@@ -105,7 +119,8 @@ oobs_object_class_init (OobsObjectClass *class)
   object_class->set_property = oobs_object_set_property;
   object_class->finalize     = oobs_object_finalize;
 
-  class->get_authentication_action = oobs_object_real_get_authentication_action;
+  /* some object types don't need to be singletons, they override this */
+  class->singleton = TRUE;
 
   dbus_connection_quark = g_quark_from_static_string ("oobs-dbus-connection");
 
@@ -149,7 +164,10 @@ oobs_object_init (OobsObject *object)
   OobsObjectPrivate *priv;
 
   priv = OOBS_OBJECT_GET_PRIVATE (object);
+
+  /* We need session to stay alive until object is destroyed */
   priv->session = oobs_session_get ();
+  g_object_ref (priv->session);
   priv->remote_object = NULL;
   dbus_error_init (&priv->dbus_error);
 
@@ -172,8 +190,7 @@ oobs_object_finalize (GObject *object)
   g_list_foreach (priv->pending_calls, (GFunc) dbus_pending_call_unref, NULL);
   g_list_free (priv->pending_calls);
 
-  _oobs_session_unregister_object (priv->session, obj);
-
+  g_object_unref (priv->session);
   g_free (priv->remote_object);
   g_free (priv->path);
   g_free (priv->method);
@@ -187,10 +204,11 @@ oobs_object_constructor (GType                  type,
 			 guint                  n_construct_properties,
 			 GObjectConstructParam *construct_params)
 {
+  OobsObjectClass *class;
   static GHashTable *type_table;
   GObject *object;
 
-  /* objects that inherit from OobsObject have to be
+  /* some objects that inherit from OobsObject have to be
    * singletons, keep a hash table with references to them
    */
   if (!type_table)
@@ -198,15 +216,22 @@ oobs_object_constructor (GType                  type,
 
   object = g_hash_table_lookup (type_table, GINT_TO_POINTER (type));
 
-  if (!object)
+  if (object)
     {
-      object = (* G_OBJECT_CLASS (oobs_object_parent_class)->constructor) (type,
-									   n_construct_properties,
-									   construct_params);
-      g_hash_table_insert (type_table, GINT_TO_POINTER (type), object);
-      connect_object_to_session (OOBS_OBJECT (object));
+      class = OOBS_OBJECT_GET_CLASS (object);
+
+      /* if has to be singleton and exists, return it */
+      if (class->singleton)
+	return object;
     }
 
+  /* this class should not be limited to singletons, create (possibly new) object anyway */
+  object = (* G_OBJECT_CLASS (oobs_object_parent_class)->constructor) (type,
+                                                                       n_construct_properties,
+                                                                       construct_params);
+  g_hash_table_insert (type_table, GINT_TO_POINTER (type), object);
+  connect_object_to_session (OOBS_OBJECT (object));
+
   return object;
 }
 
@@ -259,7 +284,6 @@ connect_object_to_session (OobsObject *object)
       return;
     }
 
-  _oobs_session_register_object (priv->session, object);
   dbus_connection_add_filter (connection, changed_signal_filter, object, NULL);
 
   rule = g_strdup_printf ("type='signal',interface='%s',path='%s'",
@@ -313,12 +337,6 @@ oobs_object_get_property (GObject      *object,
     }
 }
 
-static const gchar *
-oobs_object_real_get_authentication_action (OobsObject *object)
-{
-  return POLKIT_ACTION;
-}
-
 DBusMessage*
 _oobs_object_get_dbus_message (OobsObject *object)
 {
@@ -377,7 +395,7 @@ run_message (OobsObject  *object,
 
   if (!oobs_session_get_connected (priv->session))
     {
-      g_warning ("could send message, OobsSession hasn't connected to the bus");
+      g_warning ("Could not send message, OobsSession hasn't connected to the bus");
       return NULL;
     }
 
@@ -421,6 +439,7 @@ async_message_cb (DBusPendingCall *pending_call, gpointer data)
 	  /* FIXME: process error */
 	  result = OOBS_RESULT_MALFORMED_DATA;
 	}
+      g_warning ("There was an unknown error communicating asynchronously with the backends: %s", error.message);
 
       dbus_error_free (&error);
     }
@@ -480,11 +499,12 @@ run_message_async (OobsObject          *object,
 }
 
 static DBusMessage*
-get_commit_message (OobsObject *object)
+get_commit_message (_OobsObjectCommitMethod method, OobsObject *object)
 {
   OobsObjectClass   *class;
   OobsObjectPrivate *priv;
   DBusMessage       *message;
+  gchar *suffix;
 
   priv  = object->_priv;
   class = OOBS_OBJECT_GET_CLASS (object);
@@ -496,13 +516,30 @@ get_commit_message (OobsObject *object)
       return NULL;
     }
 
+
+  switch (method)
+    {
+      case METHOD_COMMIT:
+	suffix = "set";
+	break;
+      case METHOD_ADD:
+	suffix = "add";
+	break;
+      case METHOD_DELETE:
+	suffix = "del";
+	break;
+      default:
+	g_critical ("Unknown commit method");
+        return NULL;
+    }
+
   if (!class->commit)
     {
       g_critical ("There is no commit() implementation for this object");
       return NULL;
     }
 
-  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, "set");
+  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, suffix);
 
   /* Let the commit() implementation fill the message */
   _oobs_object_set_dbus_message (object, message);
@@ -523,8 +560,11 @@ static DBusMessage*
 get_update_message (OobsObject *object)
 {
   OobsObjectPrivate *priv;
+  OobsObjectClass *class;
+  DBusMessage *message;
 
   priv = object->_priv;
+  class = OOBS_OBJECT_GET_CLASS (object);
 
   if (!priv->session)
     {
@@ -533,60 +573,75 @@ get_update_message (OobsObject *object)
       return NULL;
     }
 
-  return dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, "get");
+  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, "get");
+
+  /* Some objects use custom messages where they need to append arguments */
+  if (class->get_update_message)
+    {
+      _oobs_object_set_dbus_message (object, message);
+      class->get_update_message (object);
+      return g_object_steal_qdata (G_OBJECT (object), dbus_connection_quark);
+    }
+  else
+    return message;
 }
 
-/**
- * oobs_object_commit:
- * @object: an #OobsObject
- * 
- * Commits to the system all the changes done
- * to the configuration held by an #OobsObject.
- *
- * Return value: an #OobsResult enum with the error code.
- **/
-OobsResult
-oobs_object_commit (OobsObject *object)
+/*
+ * Do the real work for oobs_object_commit() oobs_object_add() and oobs_object_delete().
+ */
+static OobsResult
+do_commit (_OobsObjectCommitMethod method, OobsObject *object)
 {
+  OobsObjectPrivate *priv;
   DBusMessage *message;
+  DBusMessage *reply;
+  DBusMessageIter iter;
   OobsResult result;
 
   g_return_val_if_fail (OOBS_IS_OBJECT (object), OOBS_RESULT_MALFORMED_DATA);
 
-  message = get_commit_message (object);
+  message = get_commit_message (method, object);
 
   if (!message)
     return OOBS_RESULT_MALFORMED_DATA;
 
-  run_message (object, message, &result);
+  reply = run_message (object, message, &result);
   dbus_message_unref (message);
+
+  /* Objects can use this to update themselves when the backend provides it */
+  if (reply)
+    {
+      dbus_message_iter_init (reply, &iter);
+      if (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_STRUCT)
+	{
+	  priv = object->_priv;
+
+	  priv->update_requests++;
+	  result = update_object_from_message (object, reply);
+	  dbus_message_unref (reply);
+	}
+    }
+
   g_signal_emit (object, object_signals [COMMITTED], 0);
 
   return result;
 }
 
-/**
- * oobs_object_commit_async:
- * @object: An #OobsObject.
- * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
- * @data: Aditional data to pass to @func.
- * 
- * Commits to the system all the changes done to the configuration held by an #OobsObject.
- * This change will be asynchronous, being run the function @func when the change has been done.
- * 
- * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
- * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
- **/
-OobsResult
-oobs_object_commit_async (OobsObject          *object,
-			  OobsObjectAsyncFunc  func,
-			  gpointer             data)
+/*
+ * Do the real work for oobs_object_commit_async(),
+ * oobs_object_add_async() and oobs_object_delete_async().
+ */
+static OobsResult
+do_commit_async (_OobsObjectCommitMethod method,
+                 OobsObject             *object,
+                 OobsObjectAsyncFunc     func,
+                 gpointer                data)
 {
   DBusMessage *message;
 
   g_return_val_if_fail (OOBS_IS_OBJECT (object), OOBS_RESULT_MALFORMED_DATA);
 
-  message = get_commit_message (object);
+  message = get_commit_message (method, object);
 
   if (!message)
     return OOBS_RESULT_MALFORMED_DATA;
@@ -597,10 +652,117 @@ oobs_object_commit_async (OobsObject          *object,
   return OOBS_RESULT_OK;
 }
 
+/**
+ * oobs_object_commit:
+ * @object: an #OobsObject
+ *
+ * Commits to the system all the changes done
+ * to the configuration held by an #OobsObject.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_commit (OobsObject *object)
+{
+  return do_commit (METHOD_COMMIT, object);
+}
+
+/**
+ * oobs_object_commit_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Commits to the system all the changes done to the configuration held by an #OobsObject.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_commit_async (OobsObject          *object,
+			  OobsObjectAsyncFunc  func,
+			  gpointer             data)
+{
+  return do_commit_async (METHOD_COMMIT, object, func, data);
+}
+
+/**
+ * oobs_object_add:
+ * @object: an #OobsObject
+ *
+ * Add an #OobsObject to the system's configuration. This is only
+ * supported by certain individual objects, others are added by committing
+ * a modified objects list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_add (OobsObject *object)
+{
+  return do_commit (METHOD_ADD, object);
+}
+
+/**
+ * oobs_object_add_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Add an #OobsObject to the system's configuration.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_add_async (OobsObject          *object,
+                       OobsObjectAsyncFunc  func,
+                       gpointer             data)
+{
+  return do_commit_async (METHOD_ADD, object, func, data);
+}
+
+/**
+ * oobs_object_delete:
+ * @object: an #OobsObject
+ *
+ * Delete an #OobsObject from the system's configuration. This is only
+ * supported by certain individual objects, others are added by committing
+ * a modified objects list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_delete (OobsObject *object)
+{
+  return do_commit (METHOD_DELETE, object);
+}
+
+/**
+ * oobs_object_delete_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Delete an #OobsObject from the system's configuration.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_delete_async (OobsObject          *object,
+                          OobsObjectAsyncFunc  func,
+                          gpointer             data)
+{
+  return do_commit_async (METHOD_DELETE, object, func, data);
+}
+
 /**
  * oobs_object_update:
  * @object: an #OobsObject
- * 
+ *
  * Synchronizes the configuration held by the #OobsObject
  * with the actual system configuration. All the changes done
  * to the configuration held by the #OobsObject will be forgotten.
@@ -739,21 +901,66 @@ oobs_object_ensure_update (OobsObject *object)
 }
 
 /**
- * oobs_object_get_authentication_action:
+ * oobs_object_authenticate:
  * @object: An #OobsObject.
+ * @error: Return location for error or NULL.
  *
- * Returns the PolicyKit action required to be able to modify this object configuration.
+ * Performs a PolicyKit authentication via the backends for the action
+ * required by the given object. User interaction will occur synchronously
+ * if needed.
  *
- * Return Value: string defining the PolicyKit action
- *               required to modify objects in the session.
+ * You may want to check the returned error for %OOBS_ERROR_AUTHENTICATION_CANCELLED,
+ * in which case you should avoid showing an error dialog to the user.
+ *
+ * Return Value: %TRUE if allowed to commit @object, %FALSE otherwise.
  **/
-G_CONST_RETURN gchar *
-oobs_object_get_authentication_action (OobsObject *object)
+gboolean
+oobs_object_authenticate (OobsObject *object,
+                          GError    **error)
 {
-  OobsObjectClass *class;
+  OobsObjectPrivate *priv;
+  DBusConnection    *connection;
+  DBusMessage       *message;
+  DBusMessage       *reply;
+  DBusMessageIter   iter;
+  gboolean result;
 
-  g_return_val_if_fail (OOBS_IS_OBJECT (object), NULL);
+  g_return_val_if_fail (OOBS_IS_OBJECT (object), FALSE);
+
+  priv = OOBS_OBJECT_GET_PRIVATE (object);
+
+  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path,
+                                          "org.freedesktop.SystemToolsBackends.Authentication",
+                                          "authenticate");
+
+  if (!oobs_session_get_connected (priv->session))
+    {
+      g_warning ("Could not send message, OobsSession hasn't connected to the bus");
+      return FALSE;
+    }
+
+  connection = _oobs_session_get_connection_bus (priv->session);
+  reply = dbus_connection_send_with_reply_and_block (connection, message, -1, &priv->dbus_error);
+
+  if (dbus_error_is_set (&priv->dbus_error))
+    {
+      if (dbus_error_has_name (&priv->dbus_error,
+                               "org.freedesktop.SystemToolsBackends.AuthenticationCancelled"))
+	g_set_error_literal (error, OOBS_ERROR,
+	                     OOBS_ERROR_AUTHENTICATION_CANCELLED,
+	                     priv->dbus_error.message);
+      else
+	g_set_error_literal (error, OOBS_ERROR,
+	                     OOBS_ERROR_AUTHENTICATION_FAILED,
+	                     priv->dbus_error.message);
+
+      dbus_error_free (&priv->dbus_error);
+      return FALSE;
+    }
+
+  dbus_message_iter_init (reply, &iter);
+  result = utils_get_boolean (&iter);
+
+  return result;
 
-  class = OOBS_OBJECT_GET_CLASS (object);
-  return class->get_authentication_action (object);
 }
