@@ -172,10 +172,10 @@ oobs_users_config_init (OobsUsersConfig *config)
   priv->users_list = _oobs_list_new (OOBS_TYPE_USER);
   config->_priv = priv;
 
-  priv->groups = g_hash_table_new_full (NULL, NULL, g_object_unref, NULL);
+  priv->groups = g_hash_table_new (NULL, NULL);
 }
 
-static void
+void
 free_configuration (OobsUsersConfig *config)
 {
   OobsUsersConfigPrivate *priv;
@@ -351,15 +351,15 @@ oobs_users_config_update (OobsObject *object)
       oobs_list_append (priv->users_list, &list_iter);
       oobs_list_set    (priv->users_list, &list_iter, G_OBJECT (user));
 
+      g_object_unref (user);
+
       /* keep the group name in a hashtable, this will be needed
        * each time the groups configuration changes
        */
       g_hash_table_insert (priv->groups,
-                           g_object_ref (user),
+                           user,
                            (gpointer) gid);
 
-      g_object_unref (user);
-
       dbus_message_iter_next (&elem_iter);
     }
 
@@ -420,9 +420,14 @@ oobs_users_config_commit (OobsObject *object)
 OobsObject*
 oobs_users_config_get (void)
 {
-  return g_object_new (OOBS_TYPE_USERS_CONFIG,
-		       "remote-object", USERS_CONFIG_REMOTE_OBJECT,
-		       NULL);
+  static OobsObject *the_object = NULL;
+
+  if (!the_object)
+    the_object = g_object_new (OOBS_TYPE_USERS_CONFIG,
+                               "remote-object", USERS_CONFIG_REMOTE_OBJECT,
+                               NULL);
+
+  return the_object;
 }
 
 /**
@@ -500,26 +505,52 @@ oobs_users_config_delete_user (OobsUsersConfig *config, OobsUser *user)
   OobsListIter list_iter;
   gboolean valid;
   OobsResult result;
+  OobsList *groups_list;
+  OobsGroup *group;
 
   g_return_val_if_fail (config != NULL, OOBS_RESULT_MALFORMED_DATA);
   g_return_val_if_fail (user != NULL, OOBS_RESULT_MALFORMED_DATA);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), OOBS_RESULT_MALFORMED_DATA);
   g_return_val_if_fail (OOBS_IS_USER (user), OOBS_RESULT_MALFORMED_DATA);
 
+  /* Try to commit changes */
   result = oobs_object_delete (OOBS_OBJECT (user));
 
   if (result != OOBS_RESULT_OK)
     return result;
 
+
   priv = config->_priv;
 
+  /* Remove user from all groups, to avoid committing to /etc/group
+   * the name of a non-existent user */
+  groups_list = oobs_groups_config_get_groups (OOBS_GROUPS_CONFIG (oobs_groups_config_get ()));
+
+  valid = oobs_list_get_iter_first (groups_list, &list_iter);
+
+  while (valid) {
+    group = OOBS_GROUP (oobs_list_get (groups_list, &list_iter));
+
+    oobs_group_remove_user (group, user);
+
+    g_object_unref (group);
+
+    valid = oobs_list_iter_next (priv->users_list, &list_iter);
+  }
+
+
+  /* Then remove user from the list */
   valid = oobs_list_get_iter_first (priv->users_list, &list_iter);
 
   while (valid) {
     list_user = OOBS_USER (oobs_list_get (priv->users_list, &list_iter));
 
-    if (list_user == user)
+    if (list_user == user) {
+      g_object_unref (list_user);
       break;
+    }
+
+    g_object_unref (list_user);
 
     valid = oobs_list_iter_next (priv->users_list, &list_iter);
   }
